package main

import (
	"bytes"
	"encoding/json"
	"fmt"
	"os"

	"google.golang.org/protobuf/encoding/protojson"
	"google.golang.org/protobuf/proto"

	"proto2json/pb"
)

// WriteProtobufToJSONFile writes protocol buffer message to JSON file
func WriteProtobufToJSONFile(message proto.Message, filename string) error {
	data, err := protojson.Marshal(message)
	if err != nil {
		return fmt.Errorf("cannot marshal proto message to JSON: %w", err)
	}

	var prettyJSON bytes.Buffer
	err = json.Indent(&prettyJSON, data, "", "  ")
	if err != nil {
		return fmt.Errorf("pretty json err: %w", err)
	}

	err = os.WriteFile(filename, prettyJSON.Bytes(), 0644)
	if err != nil {
		return fmt.Errorf("cannot write JSON data to file: %w", err)
	}

	return nil
}

// WriteProtobufToBinaryFile writes protocol buffer message to binary file
func WriteProtobufToBinaryFile(message proto.Message, filename string) error {
	data, err := proto.Marshal(message)
	if err != nil {
		return fmt.Errorf("cannot marshal proto message to binary: %w", err)
	}

	err = os.WriteFile(filename, data, 0644)
	if err != nil {
		return fmt.Errorf("cannot write binary data to file: %w", err)
	}

	return nil
}

// ReadProtobufFromBinaryFile reads protocol buffer message from binary file
func ReadProtobufFromBinaryFile(filename string, message proto.Message) error {
	data, err := os.ReadFile(filename)
	if err != nil {
		return fmt.Errorf("cannot read binary data from file: %w", err)
	}

	err = proto.Unmarshal(data, message)
	if err != nil {
		return fmt.Errorf("cannot unmarshal binary to proto message: %w", err)
	}

	return nil
}

// JSONToProtobufMessage converts JSON string to protocol buffer message
// func JSONToProtobufMessage(data []byte, message proto.Message) error {
// 	return protojson.Unmarshal(data, message)
// }

func main() {
	binaryFile := "./test.bin"
	jsonFile := "./test.json"

	p := pb.Person{
		Id:    1234,
		Name:  "John Doe",
		Email: "jdoe@example.com",
		Phones: []*pb.Person_PhoneNumber{
			{Number: "555-4321", Type: pb.Person_HOME},
		},
	}

	if err := WriteProtobufToBinaryFile(&p, binaryFile); err != nil {
		fmt.Println(err.Error())
		return
	} else {
		fmt.Println("write protobuf to bin file: ", binaryFile)
	}

	if err := WriteProtobufToJSONFile(&p, jsonFile); err != nil {
		fmt.Println(err.Error())
		return
	} else {
		fmt.Println("write protobuf to json file: ", jsonFile)
	}

	p2 := &pb.Person{}
	if err := ReadProtobufFromBinaryFile(binaryFile, p2); err != nil {
		fmt.Println(err.Error())
		return
	} else {
		fmt.Println(p2)
	}
}
